@@ -24,7 +24,7 @@ from time import gmtime
 import numpy as np
 
 from natural_selection import get_random_string
-from natural_selection.genetic_algorithms.operators.initialisation import initialise_population_random
+from natural_selection.genetic_algorithms.operators.initialisation import initialise_population_random, alien_spawn_default
 from natural_selection.genetic_algorithms.operators.selection import selection_elites_top_n, selection_parents_two, selection_survivors_all
 from natural_selection.genetic_algorithms.operators.crossover import crossover_two_uniform
 from natural_selection.genetic_algorithms.operators.mutation import mutation_randomize
@@ -273,6 +273,15 @@ class Chromosome:
         """
         return OrderedDict({gene.name : gene.value for gene in self.genes})
 
+    def to_list(self) -> list:
+        """
+        Helper function to convert chromosome into a Python list of the gene values.
+
+        Returns:
+            list: Values of the genes in order, as a list.
+        """
+        return [gene.value for gene in self.genes]
+
 
     def __getitem__(self, index) -> Gene:
         if isinstance(index, slice):
@@ -587,15 +596,16 @@ class Island:
     Args:
         function_params (dict): The parameters for the fitness function (default = None).
         maximise_function (bool): If True, the fitness value is maximised, False will minimise the function fitness (default = True).
-        elite_selection (Callable): Function for selecting individuals for crossover and mutation (default = None).
+        parent_selection (Callable): Function for selecting individuals for crossover (default = None).
         initialisation_function (Callable): A function for randomly creating new individuals from the given adam.
-        parent_selection (Callable): Function for selecting parents for crossover (default = None).
+        parent_combination (Callable): Function for combining parents for crossover (default = None).
         crossover_function (Callable): Function for crossover (default = None).
         mutation_function (Callable): Function for mutation (default = None).
         crossover_prob_function (Callable): Random probability function for crossover (default = None).
         mutation_prob_function (Callable): Random probability function for mutation (default = None).
         clone_function (Callable): Function for cloning (default = None).
         survivor_selection_function (Callable): Function for selecting survivors (default = None).
+        alien_spawn_function (Callable): Function for spawning new aliens during each generation (default = None).
         random_seed (int): Random seed for random and Numpy generators, set to None for no seed (default = None).
         name (str): General name for island, useful when working with multiple islands (default = None).
         verbose (bool): Print all information (default = None).
@@ -609,6 +619,7 @@ class Island:
         unique_genome (list): List of unique chromosomes.
         generation_info (list): List of dicts detailing info for every generation.
         population (list): The full population of members.
+        parents (list): All parents selected during the run.
         elites (list): All elites selected during the run.
         mutants (list): All mutants created during the run.
         children (list): All children created during the run.
@@ -631,13 +642,14 @@ class Island:
     def __init__(self, function_params : dict = None,
                  maximise_function : bool = True,
                  initialisation_function: Callable = initialise_population_random,
-                 elite_selection : Callable = selection_elites_top_n,
-                 parent_selection : Callable = selection_parents_two,
+                 parent_selection : Callable = selection_elites_top_n,
+                 parent_combination : Callable = selection_parents_two,
                  crossover_function : Callable = crossover_two_uniform,
                  mutation_function : Callable = mutation_randomize,
                  crossover_prob_function : Callable = crossover_prob_function_classic,
                  mutation_prob_function : Callable = mutation_prob_function_classic,
                  survivor_selection_function: Callable = selection_survivors_all,
+                 alien_spawn_function: Callable = alien_spawn_default,
                  clone_function : Callable = clone_classic,
                  random_seed: int = None,
                  name : str = None,
@@ -686,10 +698,10 @@ class Island:
         self.verbose_logging(f"island: maximise_function {maximise_function}")
         self._initialise = initialisation_function
         self.verbose_logging(f"island: initialisation_function {initialisation_function.__name__}")
-        self.elite_selection = elite_selection
-        self.verbose_logging(f"island: elite_selection {elite_selection.__name__}")
         self.parent_selection = parent_selection
         self.verbose_logging(f"island: parent_selection {parent_selection.__name__}")
+        self.parent_combination = parent_combination
+        self.verbose_logging(f"island: parent_combination {parent_combination.__name__}")
         self.crossover = crossover_function
         self.verbose_logging(f"island: crossover_function {crossover_function.__name__}")
         self.mutation = mutation_function
@@ -703,6 +715,9 @@ class Island:
         self.survivor_selection = survivor_selection_function
         self.verbose_logging(f"island: survivor_selection_function {survivor_selection_function.__name__}")
 
+        self.alien_spawn_function = alien_spawn_function
+        self.verbose_logging(f"island: survivor_selection_function {survivor_selection_function.__name__}")
+
         self.save_checkpoint = save_checkpoint_function
         self.verbose_logging(f"island: save_checkpoint_function {save_checkpoint_function.__name__}")
 
@@ -712,6 +727,7 @@ class Island:
         self.random_seed = random_seed
         self.verbose_logging(f"island: random_seed {random_seed}")
         self.elites = list()
+        self.parents = list()
         self.mutants = list()
         self.children = list()
         self.species_type = "def"
@@ -730,12 +746,12 @@ class Island:
         if '<lambda>' in repr(initialisation_function):
             w.warn("WARNING: 'initialisation_function' lambda can not be pickled using standard libraries.")
 
-        if '<lambda>' in repr(elite_selection):
-            w.warn("WARNING: 'elite_selection' lambda can not be pickled using standard libraries.")
-
         if '<lambda>' in repr(parent_selection):
             w.warn("WARNING: 'parent_selection' lambda can not be pickled using standard libraries.")
 
+        if '<lambda>' in repr(parent_combination):
+            w.warn("WARNING: 'parent_combination' lambda can not be pickled using standard libraries.")
+
         if '<lambda>' in repr(crossover_function):
             w.warn("WARNING: 'crossover_function' lambda can not be pickled using standard libraries.")
 
@@ -754,6 +770,9 @@ class Island:
         if '<lambda>' in repr(survivor_selection_function):
             w.warn("WARNING: 'survivor_selection_function' lambda can not be pickled using standard libraries.")
 
+        if '<lambda>' in repr(alien_spawn_function):
+            w.warn("WARNING: 'alien_spawn_function' lambda can not be pickled using standard libraries.")
+
         if logging_function and '<lambda>' in repr(logging_function):
             w.warn("WARNING: 'logging_function' lambda can not be pickled using standard libraries.")
 
@@ -770,6 +789,8 @@ class Island:
                  gene_min : Any = None,
                  mu : Any = None,
                  sig: Any = None,
+                 step_lower_bound : Any = None,
+                 step_upper_bound : Any = None,
                  choices : Iterable  = None,
                  gene_properties : dict = None):
         """
@@ -783,13 +804,15 @@ class Island:
             gene_min (Any, numeric type): Min value for random number generator (default = None).
             mu (Any, numeric type): Mean value of distribution to sample from (default = None).
             sig (Any, numeric type): Std. Dev. value of distribution to sample from (default = None).
+            step_lower_bound (Any, numeric type): For uniform stepping functions, defines lower bound of range (default = None).
+            step_upper_bound (Any, numeric type): For uniform stepping functions, defines upper bound of range (default = None).
             choices (Iterable): List of choices, categorical or not, to randomly choose from (default = None).
             gene_properties (dict): For custom random functions, extra params may be given (default = None).
         Returns:
             gene: A new Gene object.
         """
         return Gene(name=name, value=value, randomise_function=randomise_function, gene_max=gene_max, gene_min=gene_min,
-                    mu=mu, sig=sig, choices=choices, gene_properties=gene_properties)
+                    mu=mu, sig=sig, step_lower_bound=step_lower_bound, step_upper_bound=step_upper_bound, choices=choices, gene_properties=gene_properties)
 
     def create_chromosome(self,
                           genes: list = None,
@@ -930,19 +953,20 @@ class Island:
 
 
     def evolve(self, starting_generation : int = 0,
-                            n_generations : int = 5,
-                            crossover_probability : float = 0.5,
-                            mutation_probability : float = 0.25,
-                            crossover_params : dict = None,
-                            mutation_params : dict = None,
-                            elite_selection_params : dict = None,
-                            parent_selection_params: dict = None,
-                            survivor_selection_params: dict = None,
-                            criterion_function : Callable = None,
-                            criterion_params : dict = None,
-                            pre_generation_check_function : Callable = None,
-                            post_generation_check_function: Callable = None,
-                            post_evolution_function: Callable = None,
+               n_generations : int = 5,
+               crossover_probability : float = 0.5,
+               mutation_probability : float = 0.25,
+               crossover_params : dict = None,
+               mutation_params : dict = None,
+               parent_selection_params : dict = None,
+               parent_combination_params: dict = None,
+               survivor_selection_params: dict = None,
+               alien_spawn_params: dict = None,
+               criterion_function : Callable = None,
+               criterion_params : dict = None,
+               pre_generation_check_function : Callable = None,
+               post_generation_check_function: Callable = None,
+               post_evolution_function: Callable = None,
                ) -> Individual:
         """
         Starts the evolutionary run.
@@ -955,6 +979,7 @@ class Island:
             crossover_params (dict): Dict of params for custom crossover function (default = None).
             mutation_params (dict): Dict of params for custom mutation function (default = None).
             selection_params (dict): Dict of params for custom selection function (default = None).
+            alien_spawn_params (dict): Dict of params for alien spawn function (default = None).
             criterion_function (Callable): A function to evaluate if the desired criterion has been met (default = None).
             criterion_params (dict): Function parameters for criterion (default = None).
             pre_generation_check_function (Callable): A function to perform some custom pre-action at the start of every generation, with signature ``func(generation, island)`` (default = None).
@@ -975,21 +1000,26 @@ class Island:
         else:
             _mutation_params = {}
 
-        if elite_selection_params:
-            _elite_selection_params = elite_selection_params
-        else:
-            _elite_selection_params = {}
-
         if parent_selection_params:
             _parent_selection_params = parent_selection_params
         else:
             _parent_selection_params = {}
 
+        if parent_combination_params:
+            _parent_combination_params = parent_combination_params
+        else:
+            _parent_combination_params = {}
+
         if survivor_selection_params:
             _survivor_selection_params = survivor_selection_params
         else:
             _survivor_selection_params = {}
 
+        if alien_spawn_params:
+            _alien_spawn_params = alien_spawn_params
+        else:
+            _alien_spawn_params = {}
+
         def _default_criterion_function(island):
             return island.generation_count < starting_generation + n_generations - 1
 
@@ -1008,13 +1038,14 @@ class Island:
                 pre_generation_check_function(generation=g, island=self)
 
             self.__evolutionary_engine(g=g,
-                                       elite_selection_params=_elite_selection_params,
                                        parent_selection_params=_parent_selection_params,
+                                       parent_combination_params=_parent_combination_params,
                                        survivor_selection_params=_survivor_selection_params,
                                        crossover_probability=crossover_probability,
                                        mutation_probability=mutation_probability,
                                        crossover_params=_crossover_params,
-                                       mutation_params=_mutation_params)
+                                       mutation_params=_mutation_params,
+                                       alien_spawn_params=_alien_spawn_params)
             if post_generation_check_function:
                 post_generation_check_function(generation=g, island=self)
             g += 1
@@ -1054,27 +1085,28 @@ class Island:
 
     def __evolutionary_engine(self,
                               g,
-                              elite_selection_params,
                               parent_selection_params,
+                              parent_combination_params,
                               survivor_selection_params,
                               crossover_probability,
                               mutation_probability,
                               crossover_params,
-                              mutation_params):
+                              mutation_params,
+                              alien_spawn_params):
 
         if self.save_checkpoint_level == 1:
             self.save_checkpoint(event=f'evolve_pre_{g}', island=self)
 
-        elites = self.elite_selection(island=self,
-                                      individuals=self.clone(individuals=self.population, island=self),
-                                      **elite_selection_params)
-        self.verbose_logging(f"select: elites_count {len(elites)}")
+        selected_parents = self.parent_selection(island=self,
+                                       individuals=self.clone(individuals=self.population, island=self),
+                                       **parent_selection_params)
+        self.verbose_logging(f"select: selected_parents_count {len(selected_parents)}")
 
-        self.elites.append({'generation' : g, 'elites' : elites})
+        self.parents.append({'generation' : g, 'parents' : selected_parents})
 
         # Children are strictly copies or new objects seeing as the have a lineage and parents
         generation_children = list()
-        for parents in self.parent_selection(individuals=elites, island=self, **parent_selection_params):
+        for parents in self.parent_combination(individuals=selected_parents, island=self, **parent_combination_params):
             self.verbose_logging(f"select: parent_count {len(parents)}")
             if self.crossover_prob(crossover_probability=crossover_probability, island=self):
                 self.verbose_logging(f"cross: parents {[str(p) for p in parents]}")
@@ -1147,7 +1179,7 @@ class Island:
 
 
         population_fitnesses = [ind.fitness for ind in self.population]
-        elite_fitnesses = [ind.fitness for ind in elites]
+        selected_parents_fitnesses = [ind.fitness for ind in selected_parents]
 
         if len(offspring_fitnesses) > 0:
             self.generation_info.append(
@@ -1168,17 +1200,18 @@ class Island:
         self.generation_info.append(
             {
                 self.__generation_key: g,
-                self.__stat_key: 'elites',
-                self.__pop_len_key: len(elite_fitnesses),
-                self.__fitness_mean_key: np.mean(elite_fitnesses),
-                self.__fitness_std_key: np.std(elite_fitnesses),
-                self.__fitness_min_key: min(elite_fitnesses),
-                self.__fitness_max_key: max(elite_fitnesses),
-                self.__most_fit_key: selection_elites_top_n(island=self, individuals=elites, n=1)[0].name,
-                self.__least_fit_key : selection_elites_top_n(island=self, individuals=elites, n=1)[-1].name
+                self.__stat_key: 'parents',
+                self.__pop_len_key: len(selected_parents_fitnesses),
+                self.__fitness_mean_key: np.mean(selected_parents_fitnesses),
+                self.__fitness_std_key: np.std(selected_parents_fitnesses),
+                self.__fitness_min_key: min(selected_parents_fitnesses),
+                self.__fitness_max_key: max(selected_parents_fitnesses),
+                self.__most_fit_key: selection_elites_top_n(island=self, individuals=selected_parents, n=1)[0].name,
+                self.__least_fit_key : selection_elites_top_n(island=self, individuals=selected_parents, n=1)[-1].name
             }
         )
 
+
         self.verbose_logging(f"evolve: stats {self.generation_info[-1]}")
 
         self.generation_info.append(
@@ -1197,9 +1230,67 @@ class Island:
 
         self.verbose_logging(f"evolve: stats {self.generation_info[-1]}")
 
+        elites = selection_elites_top_n(island=self, individuals=self.population, n=4)
+        elite_fitnesses = [ind.fitness for ind in elites]
+        self.elites.append({'generation': g, 'elites': elites})
+
+        self.generation_info.append(
+            {
+                self.__generation_key: g,
+                self.__stat_key: 'elites',
+                self.__pop_len_key: len(elite_fitnesses),
+                self.__fitness_mean_key: np.mean(elite_fitnesses),
+                self.__fitness_std_key: np.std(elite_fitnesses),
+                self.__fitness_min_key: min(elite_fitnesses),
+                self.__fitness_max_key: max(elite_fitnesses),
+                self.__most_fit_key: selection_elites_top_n(island=self, individuals=elites, n=1)[0].name,
+                self.__least_fit_key: selection_elites_top_n(island=self, individuals=elites, n=1)[-1].name
+            }
+        )
+
+        self.verbose_logging(f"evolve: stats {self.generation_info[-1]}")
+
         for i in self.population:
             i.birthday()
 
+        new_aliens = self.alien_spawn_function(**alien_spawn_params ,island=self)
+
+        if len(new_aliens) > 0:
+            alien_fitnesses = list()
+            for alien in new_aliens:
+                if self.allow_twins:
+                    # Else, add it effectively allowing "twins" to exist
+                    self.verbose_logging(f"evolve: eval {str(alien)}")
+                    alien.evaluate(island=self, params=self.function_params)
+                    alien_fitnesses.append(alien.fitness)
+                    self.verbose_logging(f"evolve: add {str(alien)}")
+                    self.population.append(alien)
+                    self.unique_genome.append(alien.unique_genetic_code())
+                    self.__add_to_lineage(alien)
+                elif not alien.unique_genetic_code() in self.unique_genome:
+                    self.verbose_logging(f"evolve: eval {str(alien)}")
+                    alien.evaluate(island=self, params=self.function_params)
+                    alien_fitnesses.append(alien.fitness)
+                    # If we want a diverse gene pool, this must be true
+                    self.verbose_logging(f"evolve: add {str(alien)}")
+                    self.population.append(alien)
+                    self.unique_genome.append(alien.unique_genetic_code())
+                    self.__add_to_lineage(alien)
+
+            self.generation_info.append(
+                {
+                    self.__generation_key: g,
+                    self.__stat_key: 'aliens',
+                    self.__pop_len_key: len(alien_fitnesses),
+                    self.__fitness_mean_key: np.mean(alien_fitnesses),
+                    self.__fitness_std_key: np.std(alien_fitnesses),
+                    self.__fitness_min_key: min(alien_fitnesses),
+                    self.__fitness_max_key: max(alien_fitnesses),
+                    self.__most_fit_key: selection_elites_top_n(island=self, individuals=new_aliens, n=1)[0].name,
+                    self.__least_fit_key: selection_elites_top_n(island=self, individuals=new_aliens, n=1)[-1].name
+                }
+            )
+
         self.generation_count = g
 
         if self.save_checkpoint_level == 1:
