@@ -1,3 +1,3 @@
-from natural_selection.genetic_algorithms.operators.selection.elite_selection import *
+from natural_selection.genetic_algorithms.operators.selection.parent_selection import *
 from natural_selection.genetic_algorithms.operators.selection.survivor_selection import *
-from natural_selection.genetic_algorithms.operators.selection.parent_selection import *
\ No newline at end of file
+from natural_selection.genetic_algorithms.operators.selection.parent_combination import *
\ No newline at end of file
